@@ -124,6 +124,10 @@ gog_text_init_style (GogStyledObject *gso, GOStyle *style)
 		    && gog_axis_get_atype (GOG_AXIS (parent)) == GOG_AXIS_Y
 		    && style->text_layout.auto_angle)
 			style->text_layout.angle = 90.0;
+	} else if (GOG_IS_COLOR_SCALE (parent) && style->text_layout.auto_angle) {
+		gboolean horizontal;
+		g_object_get (G_OBJECT (parent), "horizontal", &horizontal, NULL);
+		style->text_layout.angle = (horizontal)? 0.0: 90.0;
 	}
 }
 
@@ -530,7 +534,7 @@ gog_reg_eqn_populate_editor (GogObject *gobj,
 			     GogDataAllocator *dalloc,
 			     GOCmdContext *cc)
 {
-	GtkWidget *w;
+	GtkWidget *w, *box;
 	GtkBuilder *gui;
 	GogRegEqn *reg_eqn = GOG_REG_EQN (gobj);
 
@@ -538,9 +542,8 @@ gog_reg_eqn_populate_editor (GogObject *gobj,
 	if (gui == NULL)
 		return;
 
-	go_editor_add_page (editor,
-			     go_gtk_builder_get_widget (gui, "reg-eqn-prefs"),
-			     _("Details"));
+	box = go_gtk_builder_get_widget (gui, "reg-eqn-prefs");
+	go_editor_add_page (editor, box, _("Details"));
 
 	w = go_gtk_builder_get_widget (gui, "show_eq");
 	g_object_set_data (G_OBJECT (w), "prop", (void*) "show-eq");
@@ -551,6 +554,15 @@ gog_reg_eqn_populate_editor (GogObject *gobj,
 	g_object_set_data (G_OBJECT (w), "prop", (void*) "show-r2");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), reg_eqn->show_r2);
 	g_signal_connect (w, "toggled", G_CALLBACK (cb_text_visibility_changed), gobj);
+	w = gtk_check_button_new_with_label (_("Rotate frame with text"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), GOG_TEXT (gobj)->rotate_frame);
+	g_signal_connect (w, "toggled", G_CALLBACK (rotate_frame_cb), gobj);
+	gtk_container_add (GTK_CONTAINER (box), w);
+	w = gtk_check_button_new_with_label (_("Rotate background with text"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), GOG_TEXT (gobj)->rotate_bg);
+	g_signal_connect (w, "toggled", G_CALLBACK (rotate_bg_cb), gobj);
+	gtk_container_add (GTK_CONTAINER (box), w);
+	gtk_widget_show_all (box);
 
 	g_object_unref (gui);
 
@@ -634,6 +646,47 @@ typedef GogOutlinedViewClass	GogTextViewClass;
 
 static GogViewClass *text_view_parent_klass;
 
+static void
+gog_text_view_natural_size (GogView *view, GogViewRequisition *req)
+{
+	GogText *text = GOG_TEXT (view->model);
+	char *str = gog_text_get_str (text);
+	PangoAttrList *pl = gog_text_get_markup (text);
+	GOGeometryAABR aabr;
+
+	req->w = req->h = 0.;
+	if (str != NULL) {
+		GOString *gostr = pl
+			? go_string_new_rich (str, -1,
+					      gog_text_get_markup (text),
+					      NULL)
+			: NULL;
+		GOStyle *style = go_style_dup (text->base.base.style);
+		double rot = fabs (style->text_layout.angle / 180 * M_PI);
+		if (rot > M_PI / 2.)
+			rot = M_PI - rot;
+	    if (text->rotate_frame)
+			style->text_layout.angle = 0.;
+		gog_renderer_push_style (view->renderer, style);
+		/* for natural size, we never force wrapping so use -1 as max width */
+		if (gostr) {
+			gog_renderer_get_gostring_AABR (view->renderer, gostr, &aabr, -1);
+			go_string_unref (gostr);
+		} else
+			gog_renderer_get_text_AABR (view->renderer, str, text->allow_markup, &aabr, -1);
+		gog_renderer_pop_style (view->renderer);
+		g_object_unref (style);
+		if (text->rotate_frame) {
+			req->w = aabr.w * cos (rot) + aabr.h * sin (rot);
+			req->h = aabr.w * sin (rot) + aabr.h * cos (rot);
+		} else {
+			req->w = aabr.w;
+			req->h = aabr.h;
+		}
+		g_free (str);
+	}
+}
+
 static void
 gog_text_view_size_request (GogView *v,
 			    GogViewRequisition const *available,
@@ -759,6 +812,7 @@ gog_text_view_class_init (GogTextViewClass *gview_klass)
 	GogViewClass *view_klass    = (GogViewClass *) gview_klass;
 
 	text_view_parent_klass = g_type_class_peek_parent (gview_klass);
+	view_klass->natural_size    = gog_text_view_natural_size;
 	view_klass->size_request    = gog_text_view_size_request;
 	view_klass->render	    = gog_text_view_render;
 }
